from typing import Optional

from cpl_core.console import Console
from cpl_query.extension import List

from py_to_uxf_core.abc.function_scanner_abc import FunctionScannerABC
from py_to_uxf_core.model.access_modifier_enum import AccessModifierEnum
from py_to_uxf_core.model.python_function import PythonFunction
from py_to_uxf_core.model.python_function_argument import PythonFunctionArgument


class FunctionScannerService(FunctionScannerABC):

    def __init__(self):
        FunctionScannerABC.__init__(self)

    def scan_line_for_function(self, line: str) -> Optional[PythonFunction]:
        # def xy():
        # def xy() -> int:
        # def xy(x: int, y: int) -> int:
        # async def xy(x: int, y: int) -> int:
        # async def _xy(x: int, y: int) -> int:
        # async def __xy(x: int, y: int) -> int:
        if not line.startswith('def ') and not line.startswith('async def '):
            return None

        line = line.replace('\t', '')
        # name
        name = line.split('def ')[1]
        name = name.split('(')[0]
        access_modifier = AccessModifierEnum.public
        args = List()
        return_type = 'None'

        # access_modifier
        if name == '__init__' or name == '__repr__' or name == '__str__':
            access_modifier = AccessModifierEnum.public
        elif name.startswith('_'):
            access_modifier = AccessModifierEnum.protected
        elif name.startswith('__'):
            access_modifier = AccessModifierEnum.private

        # args
        args_str = line.split('(')[1]
        args_str = args_str.split(')')[0]
        # multiple arguments
        if ',' in args_str:
            for argument in args_str.split(','):
                # with type
                type_str = 'any'
                arg_name = argument
                if ':' in argument:
                    type_str = argument.split(': ')[1]
                    arg_name = argument.split(': ')[0]
                # without type
                if arg_name.startswith(' '):
                    arg_name = arg_name.split(' ')[1]
                args.append(PythonFunctionArgument(arg_name, type_str))
        # single argument
        elif args_str != '':
            pass

        # return_type
        if '->' in line:
            return_type_str = line.split('-> ')[1]
            return_type = return_type_str.split(':')[0]

        return PythonFunction(access_modifier, name, args, return_type)
