from typing import Optional, Union

from cpl_core.configuration import ConfigurationABC
from cpl_core.console import Console
from cpl_query.extension import List

from py_to_uxf_core.abc.implementation_scanner_abc import ImplementationScannerABC
from py_to_uxf_core.configuration.parser_settings import ParserSettings
from py_to_uxf_core.model.class_implementation import ClassImplementation
from py_to_uxf_core.model.python_class import PythonClass


class ImplementationScannerService(ImplementationScannerABC):

    def __init__(self, configuration: ConfigurationABC):
        ImplementationScannerABC.__init__(self)
        self._parser_settings: ParserSettings = configuration.get_configuration(ParserSettings)

    def _get_implementation(self, base_name: str, sub_class: PythonClass, classes: List[PythonClass]) -> Optional[ClassImplementation]:
        base: Optional[PythonClass] = classes.where(lambda c: c.name == base_name).first_or_default()
        if base_name in self._parser_settings.ignore_class_names:
            return None

        if base is None:
            new_base = PythonClass(base_name)
            classes.append(new_base)
            return ClassImplementation(new_base, sub_class)

        return ClassImplementation(base, sub_class)

    def scan_line_for_implementation(self, line: str, classes: List[PythonClass]) -> Union[Optional[ClassImplementation], List[ClassImplementation]]:
        line = line.replace('    ', '')
        line = line.replace('\t', '')
        if line.startswith('class ') and '(' in line and ')' in line:
            sub_name = line.split('class ')[1].split('(')[0]
            subclasses = classes.where(lambda c: c.name == sub_name)
            implementations = List(ClassImplementation)
            for sub in subclasses:
                line = line.replace(' ', '')
                base_name = line.split('(')[1].split(')')[0]
                if ',' not in base_name:
                    implementation = self._get_implementation(base_name, sub, classes)
                    if implementation is not None:
                        implementations.append(implementation)
                    continue

                for name in base_name.split(','):
                    if '=' in name:
                        continue

                    implementation = self._get_implementation(name, sub, classes)
                    if implementation is not None:
                        implementation.is_first = False
                        implementations.append(implementation)
            return implementations
        return None
